@@ -40,7 +40,8 @@ import uuid
 from socket import gethostname
 from time import sleep
 
-import drmaa
+from drmaa import Session
+from drmaa.errors import InvalidJobException
 from redis import StrictRedis
 from redis.exceptions import ConnectionError as RedisConnectionError
 
@@ -53,14 +54,14 @@ if sys.version_info < (3, 0):
 
 #### Global settings ####
 # Redis settings
-REDIS_DB = 2
-REDIS_PORT = 7272
+REDIS_DB = int(os.getenv('GRID_MAP_REDIS_DB', '2'))
+REDIS_PORT = int(os.getenv('GRID_MAP_REDIS_PORT', '7272'))
 
 # Is mem_free configured properly on the cluster?
-USE_MEM_FREE = False
+USE_MEM_FREE = 'True' == os.getenv('GRID_MAP_USE_MEM_FREE', 'False')
 
 # Which queue should we use by default
-DEFAULT_QUEUE = 'all.q'
+DEFAULT_QUEUE = os.getenv('GRID_MAP_DEFAULT_QUEUE', 'all.q')
 
 
 class Job(object):
@@ -115,7 +116,6 @@ class Job(object):
         self.num_slots = num_slots
         self.mem_free = mem_free
         self.white_list = []
-        self.uniq_id = None
         self.name = name.replace(' ', '_')
         self.queue = queue
 
@@ -214,7 +214,7 @@ def _submit_jobs(jobs, uniq_id, temp_dir='/scratch', white_list=None,
     @type quiet: C{bool}
     """
 
-    session = drmaa.Session()
+    session = Session()
     session.initialize()
     jobids = []
 
@@ -240,7 +240,7 @@ def _append_job_to_session(session, job, uniq_id, job_num, temp_dir='/scratch/',
     object. Also sets job.job_id to the ID of the job on the grid.
 
     @param session: The current DRMAA session with the grid engine.
-    @type session: C{drmaa.Session}
+    @type session: C{Session}
     @param job: The Job to add to the queue.
     @type job: L{Job}
     @param uniq_id: The unique suffix for the tables corresponding to this job
@@ -312,7 +312,7 @@ def _collect_jobs(sid, jobids, joblist, redis_server, uniq_id,
                          be stored.
     @type redis_server: L{StrictRedis}
     @param wait: Wait for jobs to finish?
-    @type wait: Boolean, defaults to False
+    @type wait: bool
     @param temp_dir: Local temporary directory for storing output for an
                      individual job.
     @type temp_dir: C{basestring}
@@ -321,59 +321,75 @@ def _collect_jobs(sid, jobids, joblist, redis_server, uniq_id,
     for ix in range(len(jobids)):
         assert(jobids[ix] == joblist[ix].jobid)
 
-    s = drmaa.Session()
-    s.initialize(sid)
+    # Open DRMAA session as context manager
+    with Session(sid) as session:
 
-    if wait:
-        drmaaWait = drmaa.Session.TIMEOUT_WAIT_FOREVER
-    else:
-        drmaaWait = drmaa.Session.TIMEOUT_NO_WAIT
+        if wait:
+            drmaaWait = Session.TIMEOUT_WAIT_FOREVER
+        else:
+            drmaaWait = Session.TIMEOUT_NO_WAIT
 
-    s.synchronize(jobids, drmaaWait, True)
-    # print("success: all jobs finished", file=sys.stderr)
-    s.exit()
+        # Wait for jobs to finish
+        session.synchronize(jobids, drmaaWait, False)
 
-    # attempt to collect results
-    job_output_list = []
-    for ix, job in enumerate(joblist):
+        # attempt to collect results
+        job_output_list = []
+        for ix, job in enumerate(joblist):
 
-        log_stdout_fn = os.path.join(temp_dir, job.name + '.o' + jobids[ix])
-        log_stderr_fn = os.path.join(temp_dir, job.name + '.e' + jobids[ix])
+            log_stdout_fn = os.path.join(temp_dir, job.name + '.o' + jobids[ix])
+            log_stderr_fn = os.path.join(temp_dir, job.name + '.e' + jobids[ix])
 
-        try:
-            job_output = zload_db(redis_server, 'output{0}'.format(uniq_id),
-                                   ix)
-        except Exception as detail:
-            print(("Error while unpickling output for gridmap job {1} from" +
-                   " stored with key output_{0}_{1}").format(uniq_id, ix),
-                  file=sys.stderr)
-            print("This could caused by a problem with the cluster " +
-                  "environment, imports or environment variables.",
-                  file=sys.stderr)
-            print(("Try running `{5} -m gridmap.runner {0} {1} {2} {3} " +
-                   "{4}` to see if your job crashed before writing its " +
-                   "output.").format(uniq_id,
-                                     ix,
-                                     job.path,
-                                     temp_dir,
-                                     gethostname(),
-                                     sys.executable),
-                  file=sys.stderr)
-            print("Check log files for more information: ", file=sys.stderr)
-            print("stdout:", log_stdout_fn, file=sys.stderr)
-            print("stderr:", log_stderr_fn, file=sys.stderr)
-            print("Exception: {0}".format(detail))
-            sys.exit(2)
+            # Get the exit status and other status info about the job
+            job_info = session.wait(job.jobid, drmaaWait)
 
-        #print exceptions
-        if isinstance(job_output, Exception):
-            print("Exception encountered in job with log file:",
-                  file=sys.stderr)
-            print(log_stdout_fn, file=sys.stderr)
-            print(job_output, file=sys.stderr)
-            print(file=sys.stderr)
+            try:
+                job_output = zload_db(redis_server,
+                                      'output_{0}'.format(uniq_id),
+                                      ix)
+            except Exception as detail:
+                print(("Error while unpickling output for gridmap job {1} " +
+                       "stored with key output_{0}_{1}").format(uniq_id, ix),
+                      file=sys.stderr)
+                print("This usually happens when a job has crashed before " +
+                      "writing its output to the database.",
+                      file=sys.stderr)
+                print("\nHere is some information about the problem job:",
+                      file=sys.stderr)
+                print("stdout:", log_stdout_fn, file=sys.stderr)
+                print("stderr:", log_stderr_fn, file=sys.stderr)
+                if job_info.hasExited:
+                    print("Exit status: {0}".format(job_info.exitStatus),
+                          file=sys.stderr)
+                if job_info.hasSignal:
+                    print(("Terminating signal: " +
+                           "{0}").format(job_info.terminatedSignal),
+                          file=sys.stderr)
+                    print("Core dumped: {0}".format(job_info.hasCoreDump),
+                          file=sys.stderr)
+                print(("Job aborted before it ran: " +
+                       "{0}").format(job_info.wasAborted),
+                      file=sys.stderr)
+                print("Job resources: {0}".format(job_info.resourceUsage),
+                      file=sys.stderr)
+                try:
+                    print(("Job SGE status: " +
+                           "{0}").format(session.jobStatus(job.jobid)),
+                          file=sys.stderr)
+                except InvalidJobException:
+                    pass
+                print("Unpickling exception: {0}".format(detail),
+                      file=sys.stderr)
+                sys.exit(2)
 
-        job_output_list.append(job_output)
+            #print exceptions
+            if isinstance(job_output, Exception):
+                print("Exception encountered in job with log file:",
+                      file=sys.stderr)
+                print(log_stdout_fn, file=sys.stderr)
+                print(job_output, file=sys.stderr)
+                print(file=sys.stderr)
+
+            job_output_list.append(job_output)
 
     return job_output_list
 
@@ -422,7 +438,7 @@ def process_jobs(jobs, temp_dir='/scratch/', wait=True, white_list=None,
 
     # Save jobs to database
     for job_id, job in enumerate(jobs):
-        zsave_db(job, redis_server, 'job{0}'.format(uniq_id), job_id)
+        zsave_db(job, redis_server, 'job_{0}'.format(uniq_id), job_id)
 
     # Submit jobs to cluster
     sids, jobids = _submit_jobs(jobs, uniq_id, white_list=white_list,
@@ -436,8 +452,8 @@ def process_jobs(jobs, temp_dir='/scratch/', wait=True, white_list=None,
     assert(len(jobs) == len(job_outputs))
 
     # Delete keys from existing server or just
-    redis_server.delete(*redis_server.keys('job{0}_*'.format(uniq_id)))
-    redis_server.delete(*redis_server.keys('output{0}_*'.format(uniq_id)))
+    redis_server.delete(*redis_server.keys('job_{0}_*'.format(uniq_id)))
+    redis_server.delete(*redis_server.keys('output_{0}_*'.format(uniq_id)))
     return job_outputs
 
 
